﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;

namespace HeuristicLab.Problems.DataAnalysis.Benchmarks {
  public abstract class RegressionToyBenchmark : RegressionBenchmark {

    protected RegressionToyBenchmark() { }
    protected RegressionToyBenchmark(RegressionToyBenchmark original, Cloner cloner)
      : base(original, cloner) {
    }

    protected abstract List<List<double>> GenerateInput();

    protected abstract List<double> GenerateTarget(List<List<double>> data);

    public override IDataAnalysisProblemData GenerateProblemData() {
      List<string> varNames = new List<string>();
      varNames.Add(this.TargetVariable);
      varNames.AddRange(this.InputVariable);

      List<List<double>> dataList = GenerateInput();

      dataList.Insert(0, GenerateTarget(dataList));

      Dataset dataset = new Dataset(varNames, dataList);

      RegressionProblemData problemData = new RegressionProblemData(dataset, dataset.DoubleVariables.Skip(1), dataset.DoubleVariables.First());

      problemData.Name = "Data generated for benchmark problem \"" + this.Name + "\"";
      problemData.Description = this.Description;

      problemData.TestPartition.Start = this.TestPartition.Start;
      problemData.TestPartition.End = this.TestPartition.End;

      problemData.TrainingPartition.Start = this.TrainingPartition.Start;
      problemData.TrainingPartition.End = this.TrainingPartition.End;

      return problemData;
    }
  }
}
